#include "OpenGLHelpers.h"

#if defined(__ANDROID__)
  #ifdef NV_USE_OPENGL
    const char* pNameLibGL = "libGLESv1_CM.so";
  #else
    const char* pNameLibGLES2 = "libGLESv3.so";
  #endif
#elif defined(__QNX__)
  const char* pNameLibGLES2 = "libGLESv2.so.2";
#elif defined(__linux__)
  #ifdef NV_USE_OPENGL
    const char* pNameLibGL = "libGL.so.1";
  #else
    const char* pNameLibGLES2 = "libGLESv2.so.2";
  #endif
#endif

//-----------------------------------------------------------------------------
// DebugCallback
//-----------------------------------------------------------------------------
void APIENTRY DebugCallback(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, const GLchar* message, const GLvoid* userParam)
{
    std::vector<char> buf(length + 32);

    // Toggle this code block to configure whether messages show up only once
#if 0
    static std::set<std::string> viewMessagesOnce;
    std::string strMessage(message);
    if (viewMessagesOnce.find(strMessage) == viewMessagesOnce.end())
    {
        viewMessagesOnce.insert(strMessage);
    }
    else
    {
        return;
    }
#endif

    switch (type)
    {
    case GL_DEBUG_TYPE_ERROR:
        sprintf(&buf[0], "Error: %s\n", message);
        break;
    case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR:
        sprintf(&buf[0], "Deprecated behavior: %s\n", message);
        break;
    case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR:
        sprintf(&buf[0], "Undefined behavior: %s\n", message);
        break;
    case GL_DEBUG_TYPE_PORTABILITY:
        sprintf(&buf[0], "Portability: %s\n", message);
        break;
    case GL_DEBUG_TYPE_PERFORMANCE:
        sprintf(&buf[0], "Performance: %s\n", message);
        break;
    case GL_DEBUG_TYPE_OTHER:
    default:
        sprintf(&buf[0], "Other: %s\n", message);
        break;
    }

    NV_MESSAGE(&buf[0]);
}

//-----------------------------------------------------------------------------
// InitializeListOfLists
//-----------------------------------------------------------------------------
template <typename T> T** InitializeListOfLists(int resourceId, std::vector<uint8_t>& retvalData, Serialization::BlobProxy<uint8_t*>& dataProxy)
{
    BEGIN_DATA_SCOPE_FUNCTION();

    using namespace Serialization;
    
    DATABASE_HANDLE handle(resourceId);
    uint64_t totalBlobSize = GetDatabase().GetSize(handle);
    if (0 == totalBlobSize)
    {
        BEGIN_DATA_SCOPE();

        return NULL;
    }

    dataProxy = GetDatabase().Read<uint8_t*>(handle);

    // Extract the number of lists
    int* pNumLists = (int*)dataProxy.Get();
    int numLists = *pNumLists;

    // Point to the actual start of the pointer list
    uint64_t* pListStart = (uint64_t*)(pNumLists + 1);

    // Allocate space to store the actual pointers based on the offsets in the blob
    retvalData.resize(numLists * sizeof(T*));

    // Convert the offsets stored in the blob into pointers
    T** ppRetvalData = (T**)&retvalData[0];
    for (int i = 0; i < numLists; ++i)
    {
        BEGIN_DATA_SCOPE();

        ppRetvalData[i] = (T*)(pListStart[i] + dataProxy.Get());
    }

    return ppRetvalData;
}

// Template instantiations
template const GLvoid** InitializeListOfLists(int resourceId, std::vector<uint8_t>& retvalData, Serialization::BlobProxy<uint8_t*>& dataProxy);
template const GLchar** InitializeListOfLists(int resourceId, std::vector<uint8_t>& retvalData, Serialization::BlobProxy<uint8_t*>& dataProxy);
