#include "WGLHelpers.h"

#include "OpenGLHelpers.h"

#if !defined(_MSC_VER) || _MSC_VER >= 1700
#include <mutex>
#endif

//-----------------------------------------------------------------------------
// Lock Definition
//-----------------------------------------------------------------------------
#if defined(_MSC_VER) && _MSC_VER < 1700
namespace std
{
    class mutex
    {
    public:
        mutex()
        {
            InitializeCriticalSection(&m_cs);
        }

        ~mutex()
        {
            DeleteCriticalSection(&m_cs);
        }

        void lock()
        {
            EnterCriticalSection(&m_cs);
        }

        void unlock()
        {
            LeaveCriticalSection(&m_cs);
        }

    private:
        mutex(mutex&);
        mutex& operator=(mutex&);

        CRITICAL_SECTION m_cs;
    };

    template <typename T>
    class lock_guard
    {
    public:
        lock_guard(T& mutex)
            : m_mutex(mutex)
        {
            m_mutex.lock();
        }

        ~lock_guard()
        {
            m_mutex.unlock();
        }

    private:
        lock_guard(lock_guard&);
        lock_guard& operator=(lock_guard&);

        T& m_mutex;
    };
}
#endif

static std::mutex g_lock;

//-----------------------------------------------------------------------------
// MakeCurrent
//-----------------------------------------------------------------------------
BOOL MakeCurrent(HDC hdc, HGLRC hglrc)
{
    std::lock_guard<std::mutex> guard(g_lock);

    BOOL ret;
    NV_EXEC_GL(ret = wglMakeCurrent(hdc, hglrc));

    return ret;
}